<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;

class HierarquiaGridRepository {
    
    public function __construct(Connection $connection, EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function retornaHierarquiaGridExistente($nrtipohierarqorg, $nrestruturamrel, $nrorg){
        
        $params = array('P_NRTIPOHIERARQORG' => $nrtipohierarqorg,
                        'P_NRESTRUTURAMREL' => $nrestruturamrel,
                        'P_NRORG' => $nrorg);
        $query = "SELECT HIERARQUIAORG.* 
                        FROM HIERARQUIAORG
                   WHERE HIERARQUIAORG.NRTIPOHIERARQORG = :P_NRTIPOHIERARQORG
                     AND HIERARQUIAORG.DTINIVIGENCIA = (SELECT MAX(HO.DTINIVIGENCIA) 
                                            FROM HIERARQUIAORG HO
                                           WHERE HO.NRTIPOHIERARQORG = HIERARQUIAORG.NRTIPOHIERARQORG
                                             AND HO.NRESTRUTURAMREL = HIERARQUIAORG.NRESTRUTURAMREL
                                             AND HO.NRORG = HIERARQUIAORG.NRORG)
                     AND HIERARQUIAORG.NRESTRUTURAMREL = :P_NRESTRUTURAMREL
                     AND HIERARQUIAORG.NRORG = :P_NRORG";
                     
        return $this->connection->fetchAll($query, $params);
                     
    }
    
    public function getHierarquiaGridTree($nrestruturamrel, $nrestruturamsup, $nrtipohierarqorg, $nrtipoestrutsup, $where_placeholder, $buttonFloatingSearch){
        
        if($nrestruturamsup) $nrestruturamsup = " AND HO.NRESTRUTURAMSUP = $nrestruturamsup ";
        
        if($nrestruturamrel) $nrestruturamrel = " AND HO.NRESTRUTURAMREL = $nrestruturamrel ";

        $query = "
        SELECT * FROM (
        
            SELECT
                HO.NRHIERARQUIAORG,
                HO.NRORG,
                HO.NRTIPOHIERARQORG,
                HO.NMHIERARQUIAORG,
                TO_CHAR(DECODE(HO.NRESTRUTURAMSUP, HO.NRESTRUTURAMREL, NULL, HO.NRESTRUTURAMSUP)) AS NRESTRUTURAMSUP,
                TO_CHAR(HO.NRESTRUTURAMREL) AS NRESTRUTURAMREL,
                HO.DTINIVIGENCIA,
                HO.DTFIMVIGENCIA,
                HO.DTINCLUSAO,
                HO.DTULTATU,
                HO.NRORGINCLUSAO,
                HO.CDOPERINCLUSAO,
                HO.NRORGULTATU,
                HO.CDOPERULTATU,
                HO.IDATIVO,
                EREL.NMESTRUTURAM AS NMESTRUTURAMREL,
                DECODE(HO.NRESTRUTURAMSUP, HO.NRESTRUTURAMREL, NULL, ESUP.NMESTRUTURAM ) NMESTRUTURAMSUP,
                TH.NMTIPOHIERARQORG,
                TH.NMTIPOHIERARQORG AS NMTIPOHIERARQORG_DESCRICAO
            FROM
                HIERARQUIAORG HO 
            INNER JOIN
                TIPOHIERARQORG TH 
                    ON HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG 
            INNER JOIN
                ESTRUTURAM EREL 
                    ON HO.NRESTRUTURAMREL = EREL.NRESTRUTURAM 
                    AND (
                        HO.NRORG = EREL.NRORG
                    ) 
            LEFT JOIN
                ESTRUTURAM ESUP 
                    ON HO.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM 
                    AND (
                        HO.NRORG = ESUP.NRORG
                    ) 
            WHERE
                HO.NRORG = :P_NRORG
                AND HO.NRESTRUTURAMSUP <> HO.NRESTRUTURAMREL
                AND HO.NRTIPOHIERARQORG = :P_NRTIPOHIERARQORG
                $nrestruturamsup
                $nrestruturamrel
                AND HO.DTINIVIGENCIA    = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG
                                     WHERE DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC) 
                                       AND NRORG            = HO.NRORG
                                       AND NRESTRUTURAMREL  = HO.NRESTRUTURAMREL
                                       AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG )
        UNION 
        
            SELECT DISTINCT
                NULL AS NRHIERARQUIAORG,
                NULL AS NRORG,
                HO.NRTIPOHIERARQORG,
                TE.NMTIPOESTRUTURA AS NMHIERARQUIAORG,
                NULL AS NRESTRUTURAMSUP,
                TO_CHAR(HO.NRESTRUTURAMSUP) AS NRESTRUTURAMREL,
                NULL AS DTINIVIGENCIA,
                NULL AS DTFIMVIGENCIA,
                NULL AS DTINCLUSAO,
                NULL AS DTULTATU,
                NULL AS NRORGINCLUSAO,
                NULL AS CDOPERINCLUSAO,
                NULL AS NRORGULTATU,
                NULL AS CDOPERULTATU,
                NULL AS IDATIVO,
                ESUP.NMESTRUTURAM AS NMESTRUTURAMREL,
                NULL AS NMESTRUTURAMSUP,
                TE.NMTIPOESTRUTURA AS NMTIPOHIERARQORG,
                TE.NMTIPOESTRUTURA AS NMTIPOHIERARQORG_DESCRICAO
        
            FROM
                HIERARQUIAORG HO 
            INNER JOIN
                TIPOHIERARQORG TH 
                    ON HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG 
            INNER JOIN
                ESTRUTURAM ESUP 
                    ON HO.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM 
                    AND (
                        HO.NRORG = ESUP.NRORG
                    )
            INNER JOIN TIPOESTRUTURA TE 
                ON TE.NRTIPOESTRUTURA = ESUP.NRTIPOESTRUTURA
                    
            WHERE
                HO.NRORG = :P_NRORG
                AND (ESUP.NRTIPOESTRUTURA = :P_NRTIPOESTRUTSUP OR :P_NRTIPOESTRUTSUP IS NULL)
                $nrestruturamsup
                $nrestruturamrel
                AND HO.NRTIPOHIERARQORG = :P_NRTIPOHIERARQORG
                AND HO.DTINIVIGENCIA    = ( SELECT MAX(DTINIVIGENCIA) FROM HIERARQUIAORG
                                     WHERE DTINIVIGENCIA <= LAST_DAY(:P_DTMESCOMPETENC) 
                                       AND NRORG            = HO.NRORG
                                       AND NRESTRUTURAMREL  = HO.NRESTRUTURAMREL
                                       AND NRTIPOHIERARQORG = HO.NRTIPOHIERARQORG )            

                                          
        ) WHERE $where_placeholder $buttonFloatingSearch                               
        ";
        $params = [
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                    "P_NRTIPOESTRUTSUP" => $nrtipoestrutsup,
                    "P_NRTIPOHIERARQORG" => $nrtipohierarqorg,
                  ];
                  
        return $this->connection->fetchAll($query, $params);                  

    }
    
    public function getHierarquiaGrid($nrestruturamrel, $nrestruturamsup, $where_placeholder, $buttonFloatingSearch){
        
        if($nrestruturamsup) $nrestruturamsup = " AND HO.NRESTRUTURAMSUP = $nrestruturamsup ";
        
        if($nrestruturamrel) $nrestruturamrel = " AND HO.NRESTRUTURAMREL = $nrestruturamrel ";

        $query = "
        SELECT * FROM (
        
            SELECT
                HO.NRHIERARQUIAORG,
                HO.NRORG,
                HO.NRTIPOHIERARQORG,
                HO.NMHIERARQUIAORG,
                TO_CHAR(DECODE(HO.NRESTRUTURAMSUP,
                HO.NRESTRUTURAMREL,
                NULL,
                HO.NRESTRUTURAMSUP)) AS NRESTRUTURAMSUP,
                TO_CHAR(HO.NRESTRUTURAMREL) AS NRESTRUTURAMREL,
                HO.DTINIVIGENCIA,
                HO.DTFIMVIGENCIA,
                HO.DTINCLUSAO,
                HO.DTULTATU,
                HO.NRORGINCLUSAO,
                HO.CDOPERINCLUSAO,
                HO.NRORGULTATU,
                HO.CDOPERULTATU,
                HO.IDATIVO,
                EREL.NMESTRUTURAM AS NMESTRUTURAMREL,
                DECODE(HO.NRESTRUTURAMSUP,
                HO.NRESTRUTURAMREL,
                NULL,
                ESUP.NMESTRUTURAM ) NMESTRUTURAMSUP,
                TH.NMTIPOHIERARQORG,
                (SELECT
                    COUNT(VM.NRVINCULOM) 
                FROM
                    GPE_VINCULOM VM,
                    GPE_VINCULOH VH 
                WHERE
                    (
                        VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC 
                        OR VM.DTRESCISAOVINC IS NULL 
                    ) 
                    AND VM.NRORG = HO.NRORG 
                    AND (
                        VH.NRESTRUTLEGAL = HO.NRESTRUTURAMREL 
                        OR VH.NRESTRUTGEREN = HO.NRESTRUTURAMREL 
                    ) 
                    AND VM.NRVINCULOM = VH.NRVINCULOM 
                    AND (
                        VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC 
                        OR VM.DTRESCISAOVINC IS NULL 
                    ) 
                    AND VM.NRORG = VH.NRORG 
                    AND VH.DTMESCOMPETENC = (
                        SELECT
                            MAX(MH.DTMESCOMPETENC) 
                        FROM
                            GPE_VINCULOH MH 
                        WHERE
                            MH.NRVINCULOM = VH.NRVINCULOM 
                            AND MH.NRORG = VH.NRORG 
                            AND MH.DTMESCOMPETENC <= :P_DTMESCOMPETENC 
                    ) 
                ) AS NUMERODEVINC 
            FROM
                HIERARQUIAORG HO 
            INNER JOIN
                TIPOHIERARQORG TH 
                    ON HO.NRTIPOHIERARQORG = TH.NRTIPOHIERARQORG 
            INNER JOIN
                ESTRUTURAM EREL 
                    ON HO.NRESTRUTURAMREL = EREL.NRESTRUTURAM 
                    AND (
                        HO.NRORG = EREL.NRORG 
                    ) 
            LEFT JOIN
                ESTRUTURAM ESUP 
                    ON HO.NRESTRUTURAMSUP = ESUP.NRESTRUTURAM 
                    AND (
                        HO.NRORG = ESUP.NRORG 
                    ) 
            WHERE
                HO.NRORG = :P_NRORG 
                $nrestruturamsup
                $nrestruturamrel
                AND ( HO.DTFIMVIGENCIA >= :P_DTMESCOMPETENC
                      OR HO.DTFIMVIGENCIA IS NULL )
                                          
        ) WHERE $where_placeholder $buttonFloatingSearch                               
        ";
        $params = [
                    "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
                    "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
                  ];
    
        return $this->connection->fetchAll($query, $params);                  

    }
    
}