<?php

namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class TurnoController extends Crud {

    protected $dataSourceGetTurno = 'gpeTurno_getTurno';
    protected $dataSourceGetTurnoTodos = 'gpeTurno_getTurnoTodos';
    protected $dataSourceSetTurno = 'gpeTurno_setTurno';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRESCALATRABM', 'P_IDATIVO');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function getTurno (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetTurno);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function getTurnoTodos (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceGetTurnoTodos);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }
    
    public function setTurno (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetTurno;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                if($row['__is_new'] === true){
                    $row['DTINCLUSAO'] = $row['DTATUAL'];
                    $row['CDOPERINCLUSAO'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGINCLUSAO'] = $row['NRORGTRAB'];
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                    $row['NRTURNO'] = \HCM\Util\NovoCodigo::geraCodigo( $row['NRORGTRAB'], 'GPE_TURNO', 1, 12, true);
                } else {
                    $row['DTULTATU'] = $row['DTATUAL'];
                    $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                    $row['NRORGULTATU'] = $row['NRORGTRAB'];
                }
                $row['NRORG'] = $row['NRORGTRAB'];
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteTurno (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetTurno, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceSetTurno, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {
        
        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}