/********************************************************************************************/
/* Formato obrigatrio das triggers para controle do CACHE DE DADOS LOCAL.                  */
/*                                                                                          */
/* CREATE OR REPLACE TRIGGER DTULTATU_????                                                  */
/* BEFORE INSERT OR UPDATE OR DELETE                                                        */
/* ON TABELA FOR EACH ROW                                                                   */
/* BEGIN                                                                                    */
/*    IF INSERTING THEN                                                                     */
/*       :NEW.DTULTATU???? := SYSDATE;                                                      */
/*    END IF;                                                                               */
/*    IF UPDATING THEN                                                                      */
/*       IF (NVL(:OLD.COLUNA1,0) <> NVL(:NEW.COLUNA1,0)) OR                                 */
/*          (NVL(:OLD.COLUNA2,0) <> NVL(:NEW.COLUNA2,0)) THEN                               */
/*          :NEW.DTULTATU???? := SYSDATE;                                                   */
/*       END IF;                                                                            */
/*    END IF;                                                                               */
/*    IF DELETING THEN                                                                      */
/*       INSERT INTO DELETECACHE (NMTABLECACHE, CDFILICACHE, DSKEYCACHE, DTULTATUCACH)      */
/*                       VALUES ('TABELA', '', 'COL_CHAVE='||:OLD.COL_CHAVE||';', SYSDATE); */
/*    END IF;                                                                               */
/* END;                                                                                     */
/*                                                                                          */
/* OBS.: 1. Na condio UPDATING, devem ser testadas todas as colunas que fizerem parte da  */
/*          lista relacionada  tabela no programa GERAL.PAS.                               */
/*                                                                                          */
/*       2. Na condio DELETING, a coluna CDFILICACHE deve ser preenchida com '' para as   */
/*          tabelas comuns (exemplo: PRODUTO) e com a coluna chave que identifica a filial  */
/*          (geralmente CDFILIAL) para tabelas especficas por filial (exemplo: FILIOPER).  */
/*                                                                                          */
/********************************************************************************************/

/*============================================================================*/
/*============================================================================*/
/*============================================================================*/

/******************************************************************************/
/******************************************************************************/
/*                                                                            */
/* Trigger: ATUALIZA_APROPRIACAO                                              */
/* Tabela : BATIDAPONTO                                                       */
/*                                                                            */
/* Criao:                                                                   */
/* 05/05/2004 - Gustavo Nosseis dos Santos                                    */
/*                                                                            */
/******************************************************************************/

CREATE OR REPLACE 
TRIGGER ATUALIZA_APROPRIACAO
AFTER INSERT
ON BATIDAPONTO FOR EACH ROW

DECLARE
   V_CDOPERADOR VARCHAR2(12);
   V_CDFILIAL VARCHAR2(04);
   V_NRTAREFA VARCHAR2(10);
   V_NRPLATIVTARE VARCHAR2(05);
   V_EXISTE_TAREFA_ANDAMENTO NUMBER;
   V_EXISTE_OPERADOR NUMBER;
   
   V_NRPARCNEGOCIO NUMBER;
   V_NRATIVIDADE NUMBER;
   V_EXISTE_ATIVIDADE_ANDAMENTO NUMBER;
   
BEGIN
   IF INSERTING THEN
      IF :NEW.IDSENTBATIDA = '2' THEN
         SELECT COUNT(*)
           INTO V_EXISTE_OPERADOR
           FROM OPERADOR
          WHERE CDEMPRESA = :NEW.CDEMPRESA 
            AND CDFUNC = :NEW.CDFUNC
            AND NRORG = :NEW.NRORG;

         IF V_EXISTE_OPERADOR = 1 THEN
            SELECT MAX(CDOPERADOR), MAX(NRPARCNEGOCIO)
              INTO V_CDOPERADOR, V_NRPARCNEGOCIO
              FROM OPERADOR
             WHERE CDEMPRESA = :NEW.CDEMPRESA 
               AND CDFUNC = :NEW.CDFUNC
               AND NRORG = :NEW.NRORG;

            SELECT COUNT(*)
              INTO V_EXISTE_TAREFA_ANDAMENTO
              FROM PLANATIVTARE
             WHERE CDOPERRESP   = V_CDOPERADOR
               AND IDSITATIVTAR = 'A'
               AND NRORG = :NEW.NRORG;
               
            IF V_EXISTE_TAREFA_ANDAMENTO = 1 THEN
               SELECT CDFILIAL, NRTAREFA, NRPLATIVTARE
                 INTO V_CDFILIAL, V_NRTAREFA, V_NRPLATIVTARE
                 FROM PLANATIVTARE
                WHERE CDOPERRESP = V_CDOPERADOR
                  AND IDSITATIVTAR = 'A'
                  AND NRORG = :NEW.NRORG;

               UPDATE PLANATIVTARE
                  SET IDSITATIVTAR = 'I'
                WHERE CDFILIAL = V_CDFILIAL
                  AND NRTAREFA = V_NRTAREFA
                  AND NRPLATIVTARE = V_NRPLATIVTARE
                  AND CDOPERRESP = V_CDOPERADOR
                  AND IDSITATIVTAR = 'A'
                  AND NRORG = :NEW.NRORG;

               UPDATE APROATIVTARE
                  SET DTFAPATIVTAR = SYSDATE,
                      DTHFAPATIVTA = SYSDATE
                WHERE CDOPERADOR = V_CDOPERADOR
                  AND DTHFAPATIVTA IS NULL
                  AND CDFILIAL = V_CDFILIAL
                  AND NRTAREFA = V_NRTAREFA
                  AND NRPLATIVTARE = V_NRPLATIVTARE
                  AND NRORG = :NEW.NRORG;
            END IF;

            SELECT COUNT(*)
              INTO V_EXISTE_ATIVIDADE_ANDAMENTO
              FROM GEP_ATIVIDADE
             WHERE NRPARCOPERRESP = V_NRPARCNEGOCIO
               AND IDSITUACAOATIV = 'ANDAMENTO'
               AND NRORG = :NEW.NRORG;

            IF V_EXISTE_ATIVIDADE_ANDAMENTO = 1 THEN
               UPDATE GEP_ATIVIDADE
                  SET IDSITUACAOATIV = 'INTERROMPIDA'
                WHERE NRPARCOPERRESP = V_NRPARCNEGOCIO
                  AND IDSITUACAOATIV = 'ANDAMENTO'
                  AND NRORG = :NEW.NRORG;

               UPDATE GEP_APROPRIACAO
                  SET DTFIMAPROPRIACAO = SYSDATE,
                      DTFIMAPROPRIREAL = SYSDATE
                WHERE NRPARCOPERRESP = V_NRPARCNEGOCIO
                  AND DTFIMAPROPRIREAL IS NULL
                  AND NRORG = :NEW.NRORG;
            END IF;
         END IF;
      END IF;
   END IF;
END;
/
